/*******************************************************************************
Copyright (c) 2004 AMIS Services BV

File        : SetDefaultFindValues.java

Classes     : SetDefaultFindValues

Notes       : This Class is created by AMIS Services as complement
              to the JHeadstart 10g product.

Open Issues : The action does not check whether the number of Names and Values
              is equal. Nor can it deal with Values that contain a comma.

$revision_history$
18-may-2004   Lucas Jellema
  1.0         Created 
  
******************************************************************************/
package nl.amis.jheadstart.controller.struts.action;
// java imports
import java.util.Collection;
import java.util.HashMap;
import java.util.Locale;

import oracle.jheadstart.controller.SessionData;
import oracle.jheadstart.controller.struts.action.JhsAction;
import oracle.jheadstart.controller.struts.config.JhsActionMapping;
import oracle.jheadstart.exception.UserException;
import oracle.jheadstart.model.DataObject;
import oracle.jheadstart.model.DataObjectHandler;
import oracle.jheadstart.model.DataObjectSet;
import oracle.jheadstart.util.DataObjectHelper;
import oracle.jheadstart.view.struts.JhsDynaActionForm;

import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.DynaActionForm;


/**
  * <h3>Description</h3>
  * This action is used to set a number of session attributes from attributes on
  * a DataObject. This behavior is used for example to set default values for
  * one or more fields in a UIX FindPage. The fields in a UIX FindPage are defined 
  * as follows:
  * 
  *  <messageTextInput name="FindNAME" data:promptAndAccessKey="NAME@jheadstart:nls" data:text="FindNAME@jheadstart:sessionData" isCaseInsensitive="true" queryOperator="like" persistentAttribute="Title" maximumLength="100" columns="30" rows="1"/>
  *
  * The interesting piece here:  data:text="FindNAME@jheadstart:sessionData"
  * That means that the value displayed in the UIX page is derived from the element
  * FindNAME in the sessionData. So to set default values for find-fields in the
  * UIX findpage, we need to make sure that SessionData contains attributes with
  * the name of the Find-field. This can easily be done using this Action Class.
  *               
  * <P>
  * 
  * <h3>Action Mapping Properties</h3>
  * <table BORDER="1" CELLPADDING="3" CELLSPACING="0" COLS=6 WIDTH="100%" >
  * <tr BGCOLOR="#CCCCFF" CLASS="TableHeadingColor">
  *  <td WIDTH="1%" VALIGN="top"><b>Name</b></td>
  *  <td WIDTH="1%" VALIGN="top"><b>Type</b></td>
  *  <td WIDTH="1%" VALIGN="top"><b>Required</b></td>
  *  <td WIDTH="1%" VALIGN="top"><b>Default Value</b></td>
  *  <td WIDTH="1%" VALIGN="top"><b>Allowable Values</b></td>
  *  <td WIDTH="45%" VALIGN="top"><b>Description</b></td>
  * </tr>
  *
  * <tr BGCOLOR="white" CLASS="TableRowColor">
  * <td VALIGN="top">dataObjectName</td><td VALIGN="top">String</td>
  * <td VALIGN="top">yes</td><td VALIGN="top">&nbsp;</td><td VALIGN="top">
  * &nbsp;
  * </td><td VALIGN="top">
  * Name (key) under which the DataObject that provides the values is stored in the SessionData object.
  * </td></tr>
  *
  *
  * <tr BGCOLOR="white" CLASS="TableRowColor">
  * <td VALIGN="top">attributeNameParameter</td><td VALIGN="top">String</td>
  * <td VALIGN="top">yes</td><td VALIGN="top">&nbsp;</td><td VALIGN="top">
  * &nbsp;
  * </td><td VALIGN="top">
  * Comma-separated list of the names of the attributes to be put in the SessionData
  * by this Action.
  * Example:  &gt;set-property property="attributeNameParameter" value="FindYear,FindComplexity"/&lt;

  * </td></tr>
  * <tr BGCOLOR="white" CLASS="TableRowColor">
  * <td VALIGN="top">attributeValueParameter</td><td VALIGN="top">String</td>
  * <td VALIGN="top">yes</td><td VALIGN="top">&nbsp;</td><td VALIGN="top">
  * &nbsp;
  * </td><td VALIGN="top">
  * Comma-separated list of the names of the attributes in the DataObject that
  * provide the values for the attributes to be put in the SessionData
  * by this Action.
  * Example:  &gt;set-property property="attributeNameParameter" value="Year,Complexity"/&lt;

  * </td></tr>
  *
  * </table>
  *
  * <h3>Action Parameters</h3>
  * There are no process parameters required by this action.
  *
  * <h3>Action Results</h3>
  * The following results are stored in the SessionData by this action
  *   <ul>
  *     <li><code>[attributeNameParameter]</code>: Attributes with the names
  *     specified in this <code>attributeNameParameter</code> mapping property
  *     are stored in the SessionData.
  *     </li>
  *   </ul>
  * The scope used to store the results is taken from the <code>resultScope</code>
  * mapping property. If this property is not set, the scope is taken
  * from the (standard Struts) <code>scope</code> mapping property. If this 
  * property is not set, the scope defaults to session. 
  *
  * <h3>User Exceptions</h3>
  * The following User Exceptions might be thrown by this action
  *   <ul>
  *     <li>none</li>
  *   </ul>
  *
  * <h3>Action Forwards</h3>
  * When using this action, you should include forwards in the Struts-config
  * file for all forwards that can be returned by this action:
  *   <ul>
  *     <li><code>success</code>: set when attributes were stored
  *     in the SessionData</li>
  *     <li><code>userError</code>: set when a problem occurred, such as failure to read the sessionAttribute list</li>
  *   </ul>
  *
  * 
  * <h3>Communication with Model Layer</h3>
  * None
  *  
  * @author Lucas Jellema
  */

public class SetDefaultFindValues extends JhsAction
{


  /**
   * Start execution of action logic.
   * <p>
   * By using this method instead of the standard Struts execute
   * method we get easy access to JHeadstart Action Mappings
   * and JHeadstart SessionData as well as default handling of exceptions.
   * In addition, by coding against the SessionData interface instead of directly
   * against HttpServletRequest and HttpServletResponse, the action
   * become less Servlet specific and can easier be reused in combination
   * with future non-(http)servlet controllers.
   * @see JhsAction#execute(org.apache.struts.action.ActionMapping mapping, org.apache.struts.action.ActionForm form, javax.servlet.http.HttpServletRequest request, javax.servlet.http.HttpServletResponse response)
   */
  public ActionForward execute(JhsActionMapping mapping
                         ,SessionData sessionData 
                         ,ActionForm form) 
    throws UserException, Exception

  {
   
     // list of attribute names that should be set in the request
     String sessionAttributeNames = mapping.getAttributeNameParameter();
     // list of UserProfile DataObject Attributes that should provide the values for the request attributes to be set
     String sourceAttributeNames = mapping.getAttributeValueParameter();
     // the name of the DataObject that provides the values to be copied is specified in dataObjectName
     String dataObjectName  = mapping.getDataObjectName();
     // get the userProfile that will provide the default values
     DataObject dataObject = sessionData.getDataObject(dataObjectName);     
     setSessionAttributes(dataObject, sessionAttributeNames, sourceAttributeNames, getResultScope(mapping),sessionData);   
      
     return mapping.findForward(SUCCESS);

  } // end start

  private void setSessionAttributes (DataObject sourceDataObject , String sessionAttributeNames, String sourceAttributeNames, int scope, SessionData sessionData) throws Exception
  {   
    // we should look for the last comma, process anything before it by recursively calling
    // this method, then process what is behind the comma

    // find the first comma in both strings
    int attcommapos = sessionAttributeNames.lastIndexOf(',');
    int valuecommapos = sourceAttributeNames.lastIndexOf(',');
    if (attcommapos>-1)
    { 
      // if there is still a comma in the string, we will recursively invoke this same method to process the remainder of the string
      setSessionAttributes(sourceDataObject,  sessionAttributeNames.substring(0,attcommapos) , sourceAttributeNames.substring(0,valuecommapos), scope, sessionData);

      // strip the first segment - including the first comma - from the strings
      sessionAttributeNames = sessionAttributeNames.substring(attcommapos+1);
      sourceAttributeNames= sourceAttributeNames.substring(valuecommapos+1);
    }
    // no comma is left in the sessionAttributeNames and sourceAttributeNames
    // so we have just one source and one target attribute name left
    sessionData.setAttribute(sessionAttributeNames, sourceDataObject.getAttribute(sourceAttributeNames) , scope);
    
  }


  /**
   * Checks whether combination of properties and property values
   * specified for the action mappings of this action in the 
   * struts-config is valid.
   */
   protected void validateProperties(JhsActionMapping mapping
                                         ,SessionData sessionData 
                                         ,ActionForm form)
   {
     mapping.checkForwards(new String[]{SUCCESS, USER_ERROR});

     // check required properties   
//     mapping.checkRequiredProperty("dataObjectInterface",mapping.getDataObjectInterface());
     // name or pageName must be set
     if (mapping.getName()==null && mapping.getPageName()==null)
     {
       mapping.addInvalidPropertyException("Property name or pageName must be set");
     }
   }

}
